/*
 *= Rohde&Schwarz Digital Radiocommunication Tester Program Example ===========
 *
 * C Example Program 1: CDMA One Non Signalling Measurements
 *
 * The following example explains how to handle primary and secondary
 * addressing and how to perform simple measurement with the CMU. In the
 * example, remote control via GPIB bus and the programming language C is used.
 *
 * The following measurement example requires :
 *		CMU-B81	CDMA Link Handler
 *		CMU-K81/K82 CDMA Software package FW >= 2.50 
 *		CMU-K0		DEMO Software package
 *
 *		RSCMU200	Labwindows/CVI  driver V2.15 available at www.rsd.de (Drivers)
 *		RSCMUK8C	Labwindows/CVI  driver V2.12
 *
 * 		Optional	External RF signal generator SMIQ
 *					In that case compile project with option /D_SMIQ
 *					Settings :  - RF frequency 878.49 Mhz
 *								- Level 0.0 dBm
 *								- Digital STD IS95 STATE ON MODE REV_LINK_CODED
 *
 * Compiler name and release(s) : LabWindows/CVI 5.0
 * Original Release : January, 2001
 *
 *
 * Version : 1.0
 *
 * Copyright 2001 JM GAYET RSA. All Rights Reserved.
 *
 * ----------------------------------------------------------------------------
 *                         MODIFICATON HISTORY
 * ----------------------------------------------------------------------------
 *  Date      | Author      | Description
 *    Release |     Company |            
 * ----------------------------------------------------------------------------
 *  25.01.01  | JM Gayet	| Creation 
 *       V1.0 |         RSA | .
 *            |             | 
 *=============================================================================
 */
 
#include <ansi_c.h>
#include <utility.h>
#include <visa.h>
#include <ansi_c.h>
#include <userint.h>
#include "rscmu200.h"
#include "rscmuk8c.h"  
// #include "iec.h"


#define   	MAX_SAMPLE 						560
#define		SPAN 							3.0
#define		SPECTRUM						1
#define  	PANEL                           1
#define  	PANEL_POWER                     2 
#define  	PANEL_SPECTRUM                  3 

// Pointer to global structure
struct   RSCMU_Base_Globals   *p_globalvarBase;
   

// Macro "CheckCmuError" definition
#ifndef CheckCmuError
#define CheckCmuError(Code) if (_error = (DoCheckCmuError(__FILE__,__LINE__,Code)), (_error = (_error < 0) ? _error : 0)) \
  {goto Error;} else
#endif

// ... function definition for the previous macro
int DoCheckCmuError (char *File, int Line, ViStatus CmuStatus)
{
	ViChar l_vCmuErrMsg[1024];
	ViChar l_vErrBuf[1024];
	ViInt32 _error;
    
	if (CmuStatus != VI_SUCCESS)
	{

		//RSCMU_Base_errorMessage (VI_NULL, CmuStatus, l_vCmuErrMsg);
		RSCMU_CDMA_errorMessage(VI_NULL, CmuStatus, l_vCmuErrMsg);

		//Fmt (l_vErrBuf, "%s<Error in file %s, Line %d: %s", File, Line, l_vCmuErrMsg);
		sprintf (l_vErrBuf, "Error in file %s, Line %d: %s", File, Line, l_vCmuErrMsg);        
		MessagePopup ("ERROR MESSAGE (CMU)", l_vErrBuf);
	
	return -1;
	}
    return 0;
    
Error:
    return -1;
}

// CDMA Generator structure
typedef struct {
	ViInt32		Channel;												// Channel number 
	ViReal64	frequency;												// RF frequency
	ViReal64	freqOffset;												// Drequency Offset  
	ViReal64	PowerLev;												// Total Ouput Power 
	ViInt32		TCHChannel;												// Traffic channel number
	ViReal64	TCHLev;													// Traffic channel power level
	ViReal64	PagingLev;												// Paging  channel power level
	ViReal64	SyncLev;												// Synchro channel power level
	ViReal64	PilotLev;												// Pilot   channel power level
	ViInt32		OCNS;													// Orthogonal Code Noise 
	ViReal64	AWGNLev;
	ViInt32		PCB;
	ViInt32		FrameOffset;
	ViInt32		RATE;
	ViInt32		FrameRate;
	
} tRFGEN;


// CDMA RF Analyzer
typedef struct {
	ViReal64	Power;
	ViReal64	RHO;
	ViReal64	frequency;
	ViReal64	TIM;
	ViReal64	CarrierFeedTrough;
	ViReal64	IQ_IMB;
	ViReal64	MOT;
} tRFAN;

/************************************************************************************************/
//
// main
//
/************************************************************************************************/
int main( void ) {
 
 	// Declaration
	ViSession            	idBASE;                						// Handle with CMU base
	ViSession            	idIS95;               						// Handle with CMU CDMA
	ViSession				idRFAN;										// Handle with CMU RFAN
	ViStatus             	cmu_status;                					// Status error from driver
	ViChar               	cmu_errmsg[1024];          					// Error Message from CMU driver
	ViChar               	buffin[RD_BUFFER_SIZE_ARRAY];      			// Data buffer Input
	ViChar					buffout[WR_BUFFER_SIZE];					// Data buffer Ouput
	
	tRFGEN					CDMA_RFGEN;									// RF generator
	tRFAN					CDMA_RFAN;									// RF analyzer
	ViReal64 				RFAN_Power;									//
	ViReal64				start_freq;									//
	ViReal64 				power_trace[MAX_SAMPLE];					// Trace spectrum
	ViInt32 				NB_samples;									//
	ViInt32 				RFGEN_status;								//
	ViInt32					nb;											//
	int						_error;										//
	int						i;											//
	int						panel_handle;								//
	
	
	/************************************************************************************************/
	//
	/************************************************************************************************/
	Cls();

	printf("\nR&S CMU200 - CDMA Non Signaling  example\n");
	printf("\n\nPlease connect RF3 ouput to RF2 in if you use CMU in standalone\n");
	printf("Else connect SMIQ to RF in2 (see settings at top of the C file)\n\n");
	printf("\tOpen handle with CMU base and CMU CDMA drivers\n");
	printf("\tActive trace file\n");	
	printf("\tAssign secondary adrresses\n");		
	printf("\tSet timeout\n");	

	
	
	// Open Handle with the BASE driver
	cmu_status = RSCMU_Init_Base (&idBASE, 								// Handle to base           				
											RSCMU_GPIB, 				// Use GPIB interface       				
											0, 							//                          				
											20,							// CMU primary addresse     				
											RSCMU_DO_QUERY, 			// Query CMU identification 				
											RSCMU_RESET_ON,				// Reset CMU                				
											cmu_errmsg);
	
	// Error
	if ( cmu_status != 0) {
		printf("\nError\t: %s\n",cmu_errmsg);
		return( cmu_status );
	}
	
	// Store CMU FW version into global structure
	CheckCmuError(RSCMU_SetCmuVersion (idBASE, 
											RSCMU_200, 					// 
											"V2.50", 					// CMU FW release
											cmu_errmsg));
	

	// Active GPIB trace 
   	CheckCmuError(RSCMU_Trace_ON (idBASE, "RsCmu.iee", cmu_errmsg));
	
	// Assign secondary addresses
	CheckCmuError(RSCMU_Base_SystRemAddrSec (idBASE, 
											1, 							// Addr second. 97 
											RSCMU_RF_NSIG,  			// Map RF Analyzer
											cmu_errmsg));
	CheckCmuError(RSCMU_Base_SystRemAddrSec (idBASE, 
											2, 							// Addr second. 98
											RSCMU_CDMA800MS_NSIG, 		// Map CDMA US 800 Non signaling
							   				cmu_errmsg));
	CheckCmuError(RSCMU_Base_SystRemAddrSec (idBASE, 
											3, 							// Addr second. 99
											RSCMU_CDMA800MS_SIG, 		// Map CDMA US 800 signaling
											cmu_errmsg));
	CheckCmuError(RSCMU_Base_SystRemAddrSec (idBASE, 
											4, 							// Addr second. 100
											RSCMU_CDMA1900MS_NSIG,  	// Map CDMA PCS 1900 Non signaling
							   				cmu_errmsg));
	CheckCmuError(RSCMU_Base_SystRemAddrSec (idBASE, 
											5, 							// Addr second. 101
											RSCMU_CDMA1900MS_SIG,   	// Map CDMA PCS 1900 signaling
							   				cmu_errmsg));
							   				
	
	// Open handle with the CDMA  driver for Non signaling mode US cellular band
	CheckCmuError(RSCMU_Init_CDMA (idBASE, 								// 
											&idIS95, 					// Handle with CDMA driver                         				
											RSCMU_CDMA800MS_NSIG, 		// IS95 800 NON Signaling				
											cmu_errmsg));
	
	// Open handler with the BASIC RF analyzer
	CheckCmuError(RSCMU_Init_RF_NSig (idBASE, 
											&idRFAN, 
											cmu_errmsg));
	
	// Set Timeout base to 6 Seconds
	cmu_status = viSetAttribute (idRFAN, VI_ATTR_TMO_VALUE, 6000);
	cmu_status = viSetAttribute (idIS95, VI_ATTR_TMO_VALUE, 6000);
	
	/************************************************************************************************/
	// Select RF path Input Output
	/************************************************************************************************/
	printf("\nSelect RF path\n");
	
   	// Select RF input and output with CDMA analyzer
   	CheckCmuError(RSCMU_CDMA_InpState (idIS95, 
   											RSCMU_INPUT_RF2, 			// RF Input 2
   											cmu_errmsg));
   										
	CheckCmuError(RSCMU_CDMA_SenseCorrLossInp (idIS95, 
											RSCMU_INPUT_RF2, 
											1.0, 						// 1.0 dB compenmsation
											cmu_errmsg));
				  	
   										
   	CheckCmuError(RSCMU_CDMA_OutpState (idIS95, 
   											RSCMU_OUTPUT_RF3, 
   											cmu_errmsg));				// RF Output 3
   
	CheckCmuError(RSCMU_CDMA_SenseCorrLossOutp(idIS95, 
											RSCMU_OUTPUT_RF3, 
											0.0,						// 0.0 dB compenmsation
											cmu_errmsg));	

	// Select RF input  for the RF Analyzer
	CheckCmuError(RSCMU_RF_NSig_Inp (idRFAN, 
											RSCMU_INPUT_RF2, 			// RF Input 2
											cmu_errmsg));
											
	CheckCmuError(RSCMU_RF_NSig_SensCorrLossInp (idRFAN, 
											RSCMU_INPUT_RF2, 
											1.0,						// 1.0 dB compenmsation 
								   			cmu_errmsg));
												
	/************************************************************************************************/
	// Configure Network & Standard
	/************************************************************************************************/
	printf("\nConfigure CDMA Network US cellular & Standard IS95\n");
	
	/* Set Network & Standard */
	CheckCmuError(RSCMU_CDMA_ConfNstandard (idIS95, 
											RSCMU_CDMA_U95,				// US cellular IS95
											cmu_errmsg));
	
												
	/************************************************************************************************/
	// Configure CDMA RF Generator
	/************************************************************************************************/
	printf("\nConfigure CDMA Generator\n");	
	CDMA_RFGEN.Channel		= 283;										// Channel number
	CDMA_RFGEN.frequency	= 878.49;									// RF Forward link frequency Mhz
	CDMA_RFGEN.freqOffset	= 0.0;										// Frequency offset Khz
	CDMA_RFGEN.PowerLev		= 0.0;										// Total RF ouput power dBm
	CDMA_RFGEN.TCHChannel	= 8;										// Traffic channel
	CDMA_RFGEN.TCHLev		= -14.0;									// Traffic channel level dB
	CDMA_RFGEN.PagingLev	= -12.0;									// Paging channel level dB
	CDMA_RFGEN.SyncLev		= -16.0;									// Synchro channel level dB
	CDMA_RFGEN.PilotLev		= -7.0;										// Pilot channel level dB
	CDMA_RFGEN.OCNS			= RSCMU_CDMA_OCNS_AUTO;						// Orthogonal Code
	CDMA_RFGEN.AWGNLev		= 0.0;										// Additional Gaussian Wide Noise level dB
	CDMA_RFGEN.PCB			= RSCMU_BIT_HOLD;							// Power Bit Control
	CDMA_RFGEN.FrameOffset  = 0;										// Frame Offet
	CDMA_RFGEN.RATE			= RSCMU_CDMA_R08K;							// Frame Rate
	CDMA_RFGEN.FrameRate	= RSCMU_CDMA_FULL;							// RATE
	
	// Set the RF channel
	CheckCmuError(RSCMU_CDMA_SenseRfGChan (idIS95, 
											CDMA_RFGEN.Channel, 
											cmu_errmsg));
	// Set RF gen level	
	CheckCmuError(RSCMU_CDMA_SourceRfGLevRfl (idIS95, 
											CDMA_RFGEN.PowerLev	, 
											CDMA_RFGEN.TCHLev, 
											CDMA_RFGEN.PagingLev, 
											CDMA_RFGEN.SyncLev, 
											CDMA_RFGEN.PilotLev, 
											cmu_errmsg));
	
	// Set Power Bit Control
	CheckCmuError(RSCMU_CDMA_SourceRfGPCBits (idIS95, 
											RSCMU_BIT_HOLD,
											cmu_errmsg));
   
	
printf("\n\
		\tRF Channel\t\t: %3d\n\
		\tRF frequency\t: %6.3lf Mhz\n\
		\tRF total power\t: %6.3lf dBm\n\
		\tTraffic Channel\t: %3d\n\
		\tTraffic Level\t: %6.2lf dB\n\
		\tPaging Level\t: %6.2lf dB\n\
		\tSynchro Level\t: %6.2lf dB\n\
		\tPilot Level\t\t: %6.2lf dB\n",
		CDMA_RFGEN.Channel,
		CDMA_RFGEN.frequency,
		CDMA_RFGEN.PowerLev,
		CDMA_RFGEN.TCHChannel,
		CDMA_RFGEN.TCHLev,
		CDMA_RFGEN.PagingLev,
		CDMA_RFGEN.SyncLev,	
		CDMA_RFGEN.PilotLev);

	/************************************************************************************************/
	// Configure CDMA RF Analyzer
	/************************************************************************************************/
	
	// Set power autoranging off
	CheckCmuError(RSCMU_CDMA_SenseLevMode (idIS95, 
											RSCMU_MANUAL, 
											cmu_errmsg));
	
	// Set Power range 
	CheckCmuError(RSCMU_CDMA_SenseLevMax (idIS95, 
											CDMA_RFGEN.PowerLev + 10.0, 
											cmu_errmsg));
	// Set RF frequency
	CheckCmuError(RSCMU_CDMA_SenseRfanFreq (idIS95, 
											CDMA_RFGEN.frequency, 
											RSCMU_MHZ, 
											cmu_errmsg));

	// Set measurement consition
	CheckCmuError(RSCMU_CDMA_ConfRfanModControl (idIS95, 
											1, 
											RSCMU_SINGLE_SHOT,
								   			RSCMU_NONE, 
								   			RSCMU_NONE, 
								   			cmu_errmsg));
	
	// nb= sprintf(buffout, "%s", "CONF:MOD:CONT 1,SING,NONE,NONE;*OPC?");
	// iecin(idIS95, buffout, buffin, cmu_errmsg );



	/************************************************************************************************/
	// Configure RFAN Analyzer
	/************************************************************************************************/
	printf("\nConfigure RF Analyzer\n");
	
	// Range level max 20 dbm
	CheckCmuError(RSCMU_RF_NSig_SensLevMax (idRFAN, 
											CDMA_RFGEN.PowerLev + 10.0, 
											cmu_errmsg));
	// Autoranging OFF
	CheckCmuError(RSCMU_RF_NSig_SensLevMode (idRFAN, 
											RSCMU_MANUAL, 
											cmu_errmsg));

	// Reference level 20 dBm										
	CheckCmuError(RSCMU_RF_NSig_SensLevRef (idRFAN, 
											CDMA_RFGEN.PowerLev + 10.0, 
											cmu_errmsg));
											

	// Trigger FREE RUN										
	CheckCmuError(RSCMU_RF_NSig_TrigSour (idRFAN, 
											RSCMU_IMM, 
											cmu_errmsg));
	
	// RF Frequency										
	CheckCmuError(RSCMU_RF_NSig_RfanFreq (idRFAN, 
											CDMA_RFGEN.frequency, 
											RSCMU_MHZ, 
											cmu_errmsg));
											
	// RF filter WIDE 10 Mhz
	CheckCmuError(RSCMU_RF_NSig_RfanBandRes (idRFAN, 
											RSCMU_WIDE, 
											cmu_errmsg));
											
	// Measurement conditions
	CheckCmuError(RSCMU_RF_NSig_ConfRfanContRep (idRFAN, 
											RSCMU_SINGLE_SHOT, 
											RSCMU_NONE,
							   				RSCMU_NONE, 
							   				cmu_errmsg));
	
	
	/************************************************************************************************/
	// Configure RF Spectrum Analyzer
	/************************************************************************************************/
	printf("\nConfigure RF Spectrum\n");
	
	// Set center frequency & Span
	CheckCmuError(RSCMU_RF_NSig_SensSpecFreqCentSpan (idRFAN, 
											CDMA_RFGEN.frequency ,		// Center frequency
											RSCMU_MHZ, 					// Units
											SPAN,						// Span
											RSCMU_MHZ, 					// Units span
											cmu_errmsg));
	// Set start frequency & number of sample
	CheckCmuError(RSCMU_RF_NSig_ConfArrSpecRang (idRFAN, 
											CDMA_RFGEN.frequency - 1.5, // Frequency start 
											RSCMU_MHZ, 					// Units
											MAX_SAMPLE, 				// Number of samples to query
											cmu_errmsg));
	
	// Dynamic
	CheckCmuError(RSCMU_RF_NSig_SensSpecLevRang (idRFAN, 
											100, 						// Dynamic in dB
											cmu_errmsg));

	// configuration measurements
	CheckCmuError(RSCMU_RF_NSig_ConfSpecContRep (idRFAN, 
											RSCMU_SINGLE_SHOT, 
											RSCMU_NONE,
							   				RSCMU_NONE, 
							   				cmu_errmsg));
	


	/************************************************************************************************/
	// Start test
	/************************************************************************************************/
	
	// Stop RF analyzer
	CheckCmuError(RSCMU_RF_NSig_AborRfan (idRFAN, cmu_errmsg));
	
	// Stop CDMA RF gen
	CheckCmuError(RSCMU_CDMA_AbortRfGenerator (idIS95, cmu_errmsg));
	
	// Stop CDMA Analyzer
	CheckCmuError(RSCMU_CDMA_AbortRfanMod (idIS95, cmu_errmsg));
	

#ifndef _SMIQ

	// Init RF generator
	CheckCmuError(RSCMU_CDMA_InitRfGenerator (idIS95, cmu_errmsg));
	
	// test RF sig gen status
	CheckCmuError(RSCMU_CDMA_FetchRfGeneratorStatus (idIS95, 
											&RFGEN_status, 
											cmu_errmsg));
	if (RFGEN_status  != 1)  {
		cmu_status = -1;
		sprintf(cmu_errmsg,"Error : %d - RF generator not in state RUN", RFGEN_status);
		CheckCmuError(cmu_status);
	}

	// Loop 20 times
	//	1 - Set RF range of the CDMA RF analyzer -> Power expected + 10 dB (creast factor)
	// 	2 - Set CDMA RF power level
	//	3 - Query power measurement via CDMA RF analyzer
	//	4 - Print out
	
	for (i=0; i<20; i++) {

		CDMA_RFGEN.PowerLev = i * -1.0;
	
		// Set power expected RF power level + 10 dB (creast factor)
		CheckCmuError(RSCMU_CDMA_SenseLevMax (idIS95, 
											CDMA_RFGEN.PowerLev + 10.0, 
											cmu_errmsg));
	
	
		// Set RF gen output power 
		CheckCmuError(RSCMU_CDMA_SourceRfGLevRfl (idIS95, 
											CDMA_RFGEN.PowerLev	, 
											CDMA_RFGEN.TCHLev, 
											CDMA_RFGEN.PagingLev, 
											CDMA_RFGEN.SyncLev, 
											CDMA_RFGEN.PilotLev, 
											cmu_errmsg));
		
		// Read CDMA signal
		CheckCmuError(RSCMU_CDMA_ReadScalRfanMod (idIS95, 
											&CDMA_RFAN.Power, 
											&CDMA_RFAN.RHO,
											&CDMA_RFAN.frequency, 
											&CDMA_RFAN.TIM,
											&CDMA_RFAN.CarrierFeedTrough,
											&CDMA_RFAN.IQ_IMB, 
											&CDMA_RFAN.MOT, 
											cmu_errmsg));
													

		// Print values
		printf("\ti : %2d - BST total power : %6.2lf dBm - P Meas. = %7.3lf dBm\n", 
											i, 
											CDMA_RFGEN.PowerLev, 
											CDMA_RFAN.Power);
	

	}

#else
	//
	// This test requires an external Digital RF synthetizer. (R&S SMIQ)
	//	Simulate a CDMA IS95 reverse link
	//	Set RF total power to 0 dBm
	//
	// Loop 30 times
	//	1 - Load Graph for spectrum 
	// 	2 - Display the graph
	//	3 - Read the total power via the basic RF power meter 
	//	4 - Print power in Spectrum Panel
	//	5 - Read the trace with spectrum application
	//	6 - Plot trace in Spectrum Panel

	// Load UIR panel
	if ((panel_handle = LoadPanel (0, "rscmu_cdma_nsig_example_cvi.uir", PANEL)) < 0)
	return -1;

	
	DisplayPanel (panel_handle);
	
	printf("\nCDMA Forward power measurement\n");
	
	// Loop
	for( i= 0; i< 30; i++) {
	
		// Measure POWER with the RF Analyzer
		CheckCmuError(RSCMU_RF_NSig_ReadRfanPow (idRFAN, 
											&RFAN_Power, 
											cmu_errmsg));
	
		// Print values
		printf("\ti : %2d - P = %6.3lf dBm\n", i, RFAN_Power);
		SetCtrlVal (panel_handle, PANEL_POWER, RFAN_Power);
		
		
		// Read spectrum
		CheckCmuError(RSCMU_RF_NSig_ReadArrSpec (idRFAN, 
											&start_freq,
							   				&NB_samples, 
							   				power_trace, 
							   				cmu_errmsg));
		// Plot graph
		PlotStripChart (panel_handle, 
											PANEL_SPECTRUM ,
											power_trace, 
											MAX_SAMPLE, 
											0, 
											0,
											VAL_DOUBLE);

		
	};
	
	// 
	DiscardPanel (panel_handle);

	// Stop RF analyzer
	CheckCmuError(RSCMU_RF_NSig_AborRfan (idRFAN, cmu_errmsg));

#endif

	// Stop CDMA analyzer
	CheckCmuError(RSCMU_CDMA_AbortRfanMod (idIS95, cmu_errmsg));
	
	// Stop RF gen
	CheckCmuError(RSCMU_CDMA_AbortRfGenerator (idIS95, cmu_errmsg));
	
	// trace off
   	CheckCmuError(RSCMU_Trace_OFF (idBASE,  cmu_errmsg));

	// Close drivers
	RSCMU_CDMA_Close (idIS95, cmu_errmsg);
	RSCMU_Close_RF_NSig (idRFAN, cmu_errmsg);
	RSCMU_Close_Base (idBASE, cmu_errmsg);
	
	return 0;
Error:

    // trace off
   	CheckCmuError(RSCMU_Trace_OFF (idBASE,  cmu_errmsg));

	// Close drivers
	RSCMU_CDMA_Close (idIS95, cmu_errmsg);
	RSCMU_Close_RF_NSig (idRFAN, cmu_errmsg);
	RSCMU_Close_Base (idBASE, cmu_errmsg);
	
	
	return -1;
} /* end main */

